package com.pj.tb_wallet_topup;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.pj.project4sp.spcfg.SpCfgUtil;
import com.pj.tb_order.TbOrder;
import com.pj.tb_people.TbPeopleService;
import com.pj.tb_wallet_record.TbWalletRecord;
import com.pj.tb_wallet_record.WxPayProperties;
import com.pj.tb_wallet_record.vo.NotifyBO;
import com.pj.tb_wallet_topup.vo.ValletTopupVo;
import com.pj.utils.cache.RedisUtil;
import com.pj.utils.so.SoMap;
import com.wechat.pay.java.service.payments.app.AppServiceExtension;
import com.wechat.pay.java.service.payments.app.model.CloseOrderRequest;
import com.wechat.pay.java.service.payments.app.model.QueryOrderByIdRequest;
import com.wechat.pay.java.service.payments.app.model.QueryOrderByOutTradeNoRequest;
import com.wechat.pay.java.service.payments.model.Transaction;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * Service: tb_wallet_topup -- 服务点充值记录
 *
 * @author plj
 */
@Service
@Transactional(rollbackFor = Exception.class)
@Slf4j
public class TbWalletTopupService extends ServiceImpl<TbWalletTopupMapper, TbWalletTopup> implements IService<TbWalletTopup> {

    /**
     * 底层 Mapper 对象
     */
    @Autowired
    TbWalletTopupMapper tbWalletTopupMapper;
    @Resource
    private AppServiceExtension appServiceExtension;
    @Autowired
    WxPayProperties wxPayProperties;
    @Resource
    private TbPeopleService tbPeopleService;

    /**
     * 增
     */
    void add(TbWalletTopup t) {
        save(t);
    }

    /**
     * 删
     */
    void delete(Long id) {
        removeById(id);
    }

    /**
     * 改
     */
    void update(TbWalletTopup t) {
        updateById(t);
    }

    /**
     * 查
     */
    ValletTopupVo getById(Long id) {
        List<ValletTopupVo> listObj = tbWalletTopupMapper.getList(new SoMap().set("id", id));
        if (listObj != null && listObj.size() > 0) {
            return listObj.get(0);
        } else {
            return null;
        }
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    List<ValletTopupVo> getList(SoMap so) {
        return tbWalletTopupMapper.getList(so);
    }


    public void WxNotify(NotifyBO notifyBO) {
        String orderStatus = notifyBO.getTradeStatus();
        if (!"SUCCESS".equals(orderStatus) && !"FINISH".equals(orderStatus)) {
            log.error("支付订单回调失败:{}", JSONUtil.toJsonStr(notifyBO));
            return;
        }
        String outTradeNo = notifyBO.getOutTradeNo();
        if (StrUtil.isNotEmpty(RedisUtil.get(outTradeNo))) {
            log.error("========订单已处理==========:{}", outTradeNo);
            return;
        }
        RedisUtil.setByMINUTES(outTradeNo, DateUtil.now(), 10);
        TbWalletTopup tbWalletTopup = tbWalletTopupMapper.findByOutTradeNo(outTradeNo);
        if (tbWalletTopup == null) {
            log.error("========订单不存在==========:{}", outTradeNo);
            return;
        }
    }

    public void handlerTopup(String outTradeNoStr) {
        String key = wxPayProperties.getMchId() + "-" + wxPayProperties.getAppId();
        long now = System.currentTimeMillis() / 1000 - 10 * 60;
        String outTradeNo = StrUtil.subBefore(outTradeNoStr, "-", true);
        String timeStr = StrUtil.subAfter(outTradeNoStr, "-", true);
        if (StrUtil.isEmpty(timeStr) || now > Long.parseLong(timeStr)) {
            RedisUtil.forListRemove(key, outTradeNoStr);
            tbWalletTopupMapper.cancelTopup(outTradeNo);
            return;
        }

        QueryOrderByOutTradeNoRequest request = new QueryOrderByOutTradeNoRequest();
        // 调用request.setXxx(val)设置所需参数，具体参数可见Request定义
        request.setMchid(wxPayProperties.getMchId());
        request.setOutTradeNo(outTradeNo);
        // 调用接口
        Transaction transaction = appServiceExtension.queryOrderByOutTradeNo(request);
        if (transaction.getTradeState().name().equals("SUCCESS")) {
            RedisUtil.forListRemove(key, outTradeNoStr);
            TbWalletTopup tbWalletTopup = tbWalletTopupMapper.findByOutTradeNo(outTradeNo);
            tbWalletTopup.setPayStatus(2).setSuccessTime(transaction.getSuccessTime())
                    .setTransactionId(transaction.getTransactionId())
                    .setOpenid(transaction.getPayer().getOpenid())
                    .setUpdateTime(new Date());
            this.updateById(tbWalletTopup);
            tbPeopleService.updatePeopleWallet(tbWalletTopup.getPeopleId(), tbWalletTopup.getAmount());
        }
        log.info("Transaction:{}", transaction);
    }

    /**
     * 关闭订单
     */
    public void closeOrder(String outTradeNo) {
        CloseOrderRequest request = new CloseOrderRequest();
        // 调用request.setXxx(val)设置所需参数，具体参数可见Request定义
        request.setMchid(wxPayProperties.getMchId());
        request.setOutTradeNo(outTradeNo);
        // 调用接口
        appServiceExtension.closeOrder(request);
    }

    public TbWalletTopup getByPrePayId(String prePayId) {
        return tbWalletTopupMapper.getByPrePayId(prePayId);
    }

    /**
     * 减去扣除点数
     *
     * @param peopleId
     */
    public void subAmount(Long peopleId) {
        TbWalletTopup tbWalletTopup = getTheFirstRecord(peopleId);
        if (tbWalletTopup != null) {
            tbWalletTopup.setCutCount(tbWalletTopup.getCutCount() + SpCfgUtil.getTradeNum());
            updateById(tbWalletTopup);
        }
    }

    private TbWalletTopup getTheFirstRecord(Long peopleId) {
        LambdaQueryWrapper<TbWalletTopup> ew = new LambdaQueryWrapper<>();
        ew.eq(TbWalletTopup::getPeopleId, peopleId)
                .eq(TbWalletTopup::getPayStatus, 2)
                .apply("amount > cut_count + refund_count")
                .orderByAsc(TbWalletTopup::getCreateTime);
        List<TbWalletTopup> list = list(ew);
        return list.isEmpty() ? null : list.get(0);
    }

    public Integer getSum() {
        return tbWalletTopupMapper.getSum();
    }

    public List<TbWalletTopup> findRefundList(Long peopleId) {
        LambdaQueryWrapper<TbWalletTopup> ew = new LambdaQueryWrapper<>();
        ew.eq(TbWalletTopup::getPeopleId, peopleId)
                .eq(TbWalletTopup::getPayStatus, 2)
                .apply("amount > cut_count + refund_count")
                .orderByAsc(TbWalletTopup::getCreateTime);
        return list(ew);
    }
}
